/* This file is generated by glib-genmarshal, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */
#ifndef __MATE_PANEL_APPLET_MARSHAL_MARSHAL_H__
#define __MATE_PANEL_APPLET_MARSHAL_MARSHAL_H__

#include <glib-object.h>

G_BEGIN_DECLS

/* VOID:ENUM,BOXED,OBJECT (mate-panel-applet-marshal.list:1) */
extern
void mate_panel_applet_marshal_VOID__ENUM_BOXED_OBJECT (GClosure     *closure,
                                                        GValue       *return_value,
                                                        guint         n_param_values,
                                                        const GValue *param_values,
                                                        gpointer      invocation_hint,
                                                        gpointer      marshal_data);

/* VOID:INT (mate-panel-applet-marshal.list:2) */
#define mate_panel_applet_marshal_VOID__INT	g_cclosure_marshal_VOID__INT

/* VOID:UINT (mate-panel-applet-marshal.list:3) */
#define mate_panel_applet_marshal_VOID__UINT	g_cclosure_marshal_VOID__UINT

/* VOID:ENUM (mate-panel-applet-marshal.list:4) */
#define mate_panel_applet_marshal_VOID__ENUM	g_cclosure_marshal_VOID__ENUM

/* BOOLEAN:STRING (mate-panel-applet-marshal.list:5) */
extern
void mate_panel_applet_marshal_BOOLEAN__STRING (GClosure     *closure,
                                                GValue       *return_value,
                                                guint         n_param_values,
                                                const GValue *param_values,
                                                gpointer      invocation_hint,
                                                gpointer      marshal_data);


G_END_DECLS

#endif /* __MATE_PANEL_APPLET_MARSHAL_MARSHAL_H__ */
