
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#ifndef __PANEL_TYPEBUILTINS_H__
#define __PANEL_TYPEBUILTINS_H__ 1

#ifdef __cplusplus
extern "C" {
#endif

/* --- ../mate-panel/panel-enums-gsettings.h --- */
#define PANEL_TYPE_ORIENTATION panel_orientation_get_type()
GType panel_orientation_get_type (void);
#define PANEL_TYPE_ANIMATION_SPEED panel_animation_speed_get_type()
GType panel_animation_speed_get_type (void);
#define PANEL_TYPE_BACKGROUND_TYPE panel_background_type_get_type()
GType panel_background_type_get_type (void);
#define PANEL_TYPE_OBJECT_TYPE panel_object_type_get_type()
GType panel_object_type_get_type (void);
#define PANEL_TYPE_ACTION_BUTTON_TYPE panel_action_button_type_get_type()
GType panel_action_button_type_get_type (void);

/* --- ../mate-panel/panel-enums.h --- */
#define PANEL_TYPE_FRAME_EDGE panel_frame_edge_get_type()
GType panel_frame_edge_get_type (void);
#define PANEL_TYPE_STATE panel_state_get_type()
GType panel_state_get_type (void);
#define PANEL_TYPE_GSETTINGS_KEY_TYPE panel_gsettings_key_type_get_type()
GType panel_gsettings_key_type_get_type (void);

/* --- ../mate-panel/panel-types.h --- */
#define PANEL_TYPE_SPEED panel_speed_get_type()
GType panel_speed_get_type (void);
#ifdef __cplusplus
}
#endif

#endif /* __PANEL_TYPEBUILTINS_H__ */

/* Generated data ends here */

