
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#include <glib-object.h>
#include "mate-panel-applet-enums.h"

/* enumerations from "../libmate-panel-applet/mate-panel-applet.h" */
#include "../libmate-panel-applet/mate-panel-applet.h"
static const GEnumValue _mate_panel_applet_orient_values[] = {
  { MATE_PANEL_APPLET_ORIENT_UP, "MATE_PANEL_APPLET_ORIENT_UP", "up" },
  { MATE_PANEL_APPLET_ORIENT_DOWN, "MATE_PANEL_APPLET_ORIENT_DOWN", "down" },
  { MATE_PANEL_APPLET_ORIENT_LEFT, "MATE_PANEL_APPLET_ORIENT_LEFT", "left" },
  { MATE_PANEL_APPLET_ORIENT_RIGHT, "MATE_PANEL_APPLET_ORIENT_RIGHT", "right" },
  { 0, NULL, NULL }
};

GType
mate_panel_applet_orient_get_type (void)
{
  static GType type = 0;

  if (!type)
    type = g_enum_register_static ("MatePanelAppletOrient", _mate_panel_applet_orient_values);

  return type;
}
static const GEnumValue _mate_panel_applet_background_type_values[] = {
  { PANEL_NO_BACKGROUND, "PANEL_NO_BACKGROUND", "no-background" },
  { PANEL_COLOR_BACKGROUND, "PANEL_COLOR_BACKGROUND", "color-background" },
  { PANEL_PIXMAP_BACKGROUND, "PANEL_PIXMAP_BACKGROUND", "pixmap-background" },
  { 0, NULL, NULL }
};

GType
mate_panel_applet_background_type_get_type (void)
{
  static GType type = 0;

  if (!type)
    type = g_enum_register_static ("MatePanelAppletBackgroundType", _mate_panel_applet_background_type_values);

  return type;
}
static const GFlagsValue _mate_panel_applet_flags_values[] = {
  { MATE_PANEL_APPLET_FLAGS_NONE, "MATE_PANEL_APPLET_FLAGS_NONE", "flags-none" },
  { MATE_PANEL_APPLET_EXPAND_MAJOR, "MATE_PANEL_APPLET_EXPAND_MAJOR", "expand-major" },
  { MATE_PANEL_APPLET_EXPAND_MINOR, "MATE_PANEL_APPLET_EXPAND_MINOR", "expand-minor" },
  { MATE_PANEL_APPLET_HAS_HANDLE, "MATE_PANEL_APPLET_HAS_HANDLE", "has-handle" },
  { 0, NULL, NULL }
};

GType
mate_panel_applet_flags_get_type (void)
{
  static GType type = 0;

  if (!type)
    type = g_flags_register_static ("MatePanelAppletFlags", _mate_panel_applet_flags_values);

  return type;
}

/* Generated data ends here */

